# -*- coding: utf-8 -*-
from project_root.dku_utils.core import get_current_project_and_variables
from project_root.config.flow.connections import CONNECTIONS_REQUIRING_SYNCING_PYTHON_OUTPUTS
import dataiku
import pandas as pd, numpy as np
from dataiku import pandasutils as pdu
import math

# Read recipe inputs
project, variables = get_current_project_and_variables()
connection_type = variables["local"].get("connection_type")

if connection_type in CONNECTIONS_REQUIRING_SYNCING_PYTHON_OUTPUTS:
    objective_value_dataset_name = "objective_value_synced"
else:
    objective_value_dataset_name = "objective_value"
objective_value_prepared = dataiku.Dataset(objective_value_dataset_name)
objective_value_prepared_df = objective_value_prepared.get_dataframe()


# Compute recipe outputs
# TODO: Write here your actual code that computes the outputs
# NB: DSS supports several kinds of APIs for reading and writing data. Please see doc.


#### GET THE OBJECTIVE VALUE FROM THE OPTIMIZER
new_objective_value = math.ceil(objective_value_prepared_df['objective_value'][0])


#### WRITE THE OBJECTIVE VALUE TO THE DATAFRAME
objective_value_information_df = pd.DataFrame([[new_objective_value]], columns=['values'], index=['optimised cost'])

benchmark_calculation_bool = (dataiku.get_custom_variables()["leverage_allocation_and_procurement_baselines_app"] == "true")

#### IF BENCHMARK CALCULATION IS ENABLED, THEN CALCULATED COST REDUCTION AND ATTACH IT TO THE `objective_value_information_df` DATAFRAME
if benchmark_calculation_bool:
    # Dataset benchmark_objective_cost renamed to baseline_objective_cost by admin on 2023-03-08 16:56:56
    benchmark_objective_cost = dataiku.Dataset("baseline_objective_cost")
    benchmark_objective_cost_df = benchmark_objective_cost.get_dataframe()
    benchmark_objective_value = math.ceil(benchmark_objective_cost_df['final_cost_amount_sum'][0])
    cost_reduction_rate = (new_objective_value - benchmark_objective_value)/benchmark_objective_value * 100
    
    improvement_rate_info_df = pd.DataFrame([[benchmark_objective_value], [cost_reduction_rate]], columns=['values'], index=['old cost', 'cost_reduction_percent'])
    
    objective_value_information_df = pd.concat([objective_value_information_df, improvement_rate_info_df])

objective_value_information_df = objective_value_information_df.T
 # Compute a Pandas dataframe to write into benchmark_improvement_rate
benchmark_improvement_rate_df = objective_value_information_df.reset_index()
# Write recipe outputs
benchmark_improvement_rate = dataiku.Dataset("benchmark_improvement_rate")
benchmark_improvement_rate.write_with_schema(benchmark_improvement_rate_df)
