from project_root.inventory_optimisation_gd import pipeline
import dataiku
import pandas as pd, numpy as np
from dataiku import pandasutils as pdu
import json
from project_root.config.flow.constants import DATASET_SCHEMA




# Read recipe inputs

demand_forecast = dataiku.Dataset("demand_forecast_prepared")
demand_forecast_df = demand_forecast.get_dataframe()

shipping = dataiku.Dataset("shipping_prepared")
shipping_df = shipping.get_dataframe()

supply = dataiku.Dataset("supply_prepared")
supply_df = supply.get_dataframe()

supply_nodes = dataiku.Dataset("supply_nodes_prepared")
supply_nodes_df = supply_nodes.get_dataframe()


product_is_itemsets = (dataiku.get_custom_variables()["enable_itemset_option_app"] == "true")

itemsets_df = pd.DataFrame(demand_forecast_df['itemset_id'])
itemsets_df['sku_id'] = itemsets_df['itemset_id'].copy()

if product_is_itemsets: 
    itemsets_df['sku_id'] = itemsets_df['sku_id'].str.replace('[', '').str.replace(']', '').str.replace(' ', '')
    itemsets_df['sku_id'] = itemsets_df['sku_id'].str.split(',')
    itemsets_df = itemsets_df.explode('sku_id')
    itemsets_df = itemsets_df.reset_index(drop=True)
    
itemsets_df.drop_duplicates(inplace=True) 


# Compute recipe outputs
# TODO: Write here your actual code that computes the outputs
# NB: DSS supports several kinds of APIs for reading and writing data. Please see doc.


config = {}

config['solver'] = json.loads(dataiku.get_custom_variables()['solver_app'])
config['schema'] = DATASET_SCHEMA



config['solver']['mip_gap'] = float(dataiku.get_custom_variables()["relative_mip_gap"])



### insert data into solver pipeline
solution = pipeline(
    shipping_df, 
    itemsets_df, 
    supply_nodes_df, 
    supply_df, 
    demand_forecast_df,
    config
    )


df_warehouse_store_qty = solution.df_warehouse_store_qty
df_warehouse_sku_proc = solution.df_warehouse_sku_proc
df_warehouse_sku_qty = solution.df_warehouse_sku_qty
objective_value_solution = solution.cost





# Write recipe outputs
itemset_outputs = dataiku.Dataset("itemset_outputs")
itemset_outputs.write_with_schema(df_warehouse_store_qty)

sku_procurement_outputs = dataiku.Dataset("sku_procurement_outputs")
sku_procurement_outputs.write_with_schema(df_warehouse_sku_proc)

sku_warehouse_quantities = dataiku.Dataset("sku_warehouse_quantities")
sku_warehouse_quantities.write_with_schema(df_warehouse_sku_qty)

objective_value = dataiku.Dataset('objective_value')
objective_value.write_with_schema(pd.DataFrame({'objective_value' : [objective_value_solution]}))
