# -*- coding: utf-8 -*-
import dataiku
import pandas as pd, numpy as np
from dataiku import pandasutils as pdu

from project_root.dku_utils.core import get_current_project_and_variables
from project_root.dku_utils.recipes.sync_recipe import update_and_run_sync_recipe
from project_root.config.flow.connections import CONNECTIONS_REQUIRING_SYNCING_PYTHON_OUTPUTS


# Read recipe inputs
project, variables = get_current_project_and_variables()
connection_type = variables["local"].get("connection_type")

if connection_type in CONNECTIONS_REQUIRING_SYNCING_PYTHON_OUTPUTS:
    itemset_outputs_dataset_name = "itemset_outputs_synced"
else:
    itemset_outputs_dataset_name = "itemset_outputs"

itemset_outputs = dataiku.Dataset(itemset_outputs_dataset_name)
itemset_outputs_df = itemset_outputs.get_dataframe()


# Compute recipe outputs from inputs
# TODO: Replace this part by your actual code that computes the output, as a Pandas dataframe
# NB: DSS also supports other kinds of APIs for reading and writing data. Please see doc.

product_allocation_not_grouped_df = itemset_outputs_df # For this sample code, simply copy input to output


rename_dict = {
    'supply_node_id' : 'warehouse_id',
    'demand_node_id' : 'store_id',
    'itemset_id' : 'product_id',
    'product_stock_quantity': 'product_stock_quantity'
}


product_is_itemsets = (dataiku.get_custom_variables()["enable_itemset_option_app"] == "true")

if product_is_itemsets:
    product_allocation_not_grouped_df['itemset_id'] = product_allocation_not_grouped_df['itemset_id'].str.replace('[', '').str.replace(']', '').str.replace(' ', '')
    product_allocation_not_grouped_df['itemset_id'] = product_allocation_not_grouped_df['itemset_id'].str.split(',')
    product_allocation_not_grouped_df = product_allocation_not_grouped_df.explode('itemset_id')
    product_allocation_not_grouped_df = product_allocation_not_grouped_df.reset_index(drop=True)
    
    
product_allocation_not_grouped_df = product_allocation_not_grouped_df.rename(columns=rename_dict)



# Write recipe outputs
product_allocation_not_grouped = dataiku.Dataset("product_allocation_not_grouped")
product_allocation_not_grouped.write_with_schema(product_allocation_not_grouped_df)








