This case study offers a walkthrough of the solution using synthetically generated data for the Inventory Allocation Optimization Dataiku solution.

----


The goal of the solution is to generate the logistics plan for product procurement to the warehouses and product shipping from warehouses to stores. To test the application, you'll need a database with demo data uploaded (see  _[test_data.zip](QUlAGgHBfhYD)_  in the attachments, look at the folder  _with_itemset_data_ ). Here is the [link](https://doc.dataiku.com/dss/latest/connecting/sql/introduction.html) on how to set up a connection on Dataiku. 


## How to Run?

### Step 1: Connection settings

Choose your main connection. This dropdown will list all of your available connections; choose a connector connected to your database with demo data. After that, choose a connection type. This means selecting which database type you are using (PostgreSQL, Snowflake, etc.).

![connection_settings](oPOkxCew1W3z)

Click reconfigure and wait for the process to finish.


### Step 2: Data Specifications

Here you should enter the names of tables located in your database. For example, if your table name is for  _supply_nodes_, then input in the text field just _supply_nodes_. In case you created another project in Dataiku and want to import data from there, then use the project key as a prefix before the table name. If the project key is, for example, "_test_project_dataiku_, "then you should enter " _test_project_dataiku_supply_nodes_. "

#### Optional:

Check the "Leverate allocation and procurement baseline?" if you want to import baseline datasets to make a detailed comparison (you can find the examples in  _[test_data.zip](QUlAGgHBfhYD)_  for a test run). 


![data_specifications](adXhNj0uXX6V)

After you have specified the dataset paths, click _Load_ and after the process finishes, click  _Refresh_ to reload the page. To find more details about data source requirements, go to [Requirements](article:9).



### Step 3: Column identification

The  _Columns identification_ section lets you identify the primary columns playing a role in the Inventory Allocation Optimization so that we can process their information in the flow.

![columns_identification](ShApa7at4MPD)

If you are using test data, then use the instructions below to setup the columns:

- The  _"Supply nodes identifier column"_  is named as  _"warehouse_id"_ 
- The  _"Demand nodes identifier column"_  is named as  _"forecast granularity"_ 
- The _"Location capacity identified column"_ is named as  _"location capacity"_ 
- The _"Product identifier column"_ is named as  _"product_id"_ 
- The  _"Product shipping cost identified column"_  is named as  _"product_shipping_cost"_ 
- The  _"Product stock in warehouse identifier column"_  is named as  _"product_stock_in_warehouses"_ 
- The  _"Product unit procurement cost identifier column"_  is named as  _"product_unit_procurement_cost"_ 
- The  _"Forecasted demand identifier column"_  is named as  _"forecasted_demand"_ 
- The  _"Allocation baseline stock quantity identfiier column"_  is named as  _"product_stock_quantitiy"_ 
- The  _"Product stock procurement quantity identifier column"_  is named as  _"product_stock_procurement_quantity"_ 

### Step 4: Run the App

The final step is to run the app. If you've been following the demo, no changes in this section are needed. Leave the MIP values as 0.05 and the checkbox in  _Optimization options_ checked (we use product set instead of products).

Just click Run and wait for it to finish. 


To find more about other options that go beyond this demo, go to [Results and optimization settings](article:31) and [What is MIP](article:19).

![settings](2IOl5g8GRDHg)



## Insights

After the app finishes running, two types of results will be available. 

- Results in the CSV format (downloadable dataset)
  - [product_allocation](dataset:product_allocation)
  - [product_procurement_list](dataset:product_procurement_list)
- Insights in the dashboard

Each of these options has its own purpose; direct CSV format can be useful for a supply-chain manager to understand what product he has to procure and where they should be shipped. On the other hand, insights (charts) in the dashboard give information about the data in an aggregated format, for example, what are the total costs of shipments and procurements, what products are often procured, what are the summed up costs of products, etc.


#### Interpretation of downloadable datasets


 [product_allocation](dataset:product_allocation) dataset shows the list of shipments needed to be made. For example, the first row from the screenshot below shows that a product  `FOODS_1_228` needs to be shipped `2` quantities from the warehouse `warehouse_arizona_1` to `store_arizona_1`.
 
 ![product_allocation](gjCinujuuMEg)
 
 
 [product_procurement_list](dataset:product_procurement_list) dataset show the list of procurement needed to be made. For example, the first row from the screenshot below shows that we need to procure `11773` of `FOODS_2_435` to the warehouse named `warehouse_california_1`.
 
 
![product_procurement](uNYMPq3cOByq)


These two datasets are the main results from the solver based on minimizing the cost function mentioned in the [Optimization approach](article:12). 


#### Insights in the dashboard.

In dashboard insights you can find more of an aggregated form of solver results rather than a whole list. Here you can see the statistics of warehouse capacities, shipping costs, procurement costs, and other details. Broad details can be found at [Dashboards](article:6).
As is described in [Results Dashboard](article:33), we have detailed information about Warehouse, Shipping, and Procurement. 
The warehouse tab helps to understand the distribution of product quantities by warehouse and warehouse capacity comparison before and after optimization. This tab tracks metrics such as the number of distinct products and the total number of products. The picture below shows an example of how the interactive dashboard helps to analyze the dataset. The example shows the number of product *FOODS_2_968* across different warehouses. In this case, it is *1134* units in *warehouse_california_1*.

![warehouse_example](8k99EU5QTZRt)

The pivot table below provides a more detailed view of the distribution of the product quantities across different warehouses after optimization. For instance, product *FOODS_1_228*  is located in *warehouse_arizona_2* in the quantity of *20686*.

![warehouse_example2](5qh0I1MDQH2o)

The shipping tab gives information about shipping costs by store and by warehouse. The example below shows that the majority of shipping costs go from *warehouse_arizona_2* (in terms of warehouses, according to the donut plot) and store *store_california_9* (in terms of stores, according to the vertical bars plot).

![shipping_example](8ApnsRTnPbJo)

Additionally, the Shipping tab provides costs from different warehouses to different stores. The pivot table below demonstrates such information. For instance, *0* costs were spent to deliver goods from *warehouse_arizona_1* , *warehouse_arizona_2*, and *warehouse_nevada_2*  to the *store_california_1*. 

![shipping_example2](x25K5qIYRl1A)

The Procurement tab provides a *Procurement cost* metric as well as an extensive descriptive analysis of procurement costs by warehouse and per product. An illustration of this can be found on the plots below. The majority of procurement happened in the *warehouse_california_1*, and the most popular product in terms of procurement was *FOODS_3_104*.

![procurement_example](IGb67OeTSVQt)

![procurement_example2](0N1HwXuNjymt)


All this information is helpful for the supply chain managers to plan the procurement of products for different warehouses as well as product allocation. 
In the event that a manager would like to compare existing product allocation and procurement with new optimization results, we propose [Benchmark Dashboard](article:34). The main metric we are comparing is total costs (sum of shipping and procurement costs). In our example, the total cost is reduced by *2.491%* after optimization.

![baseline2](D7ZTk86ISUzF)

We compare a couple of major metrics in this dashboard. As an example of such comparison, we provide the *Procurement costs comparison* section of the *Benchmark* slide. Vertical bars plot compares procurement costs before and after optimization per warehouse.  The pivot table shows average improvement in terms of costs. For instance, in the best case, we have *0.048%* of cost reduction for a given product and warehouse.

![baseline](fEfTUjM9ICIL)

## Conclusion

In conclusion, the Inventory Allocation Optimization solution can be an essential tool for a supply chain manager to streamline the procurement and allocation of resources. By leveraging the Dataiku platform and mixed-integer programming, this solution can help supply chain managers make informed decisions and optimize their operations to improve efficiency and reduce costs.
Dashboards in this solution can be a powerful tool for the supply chain manager to gain real-time insights and visualize key performance indicators (KPIs) crucial for managing their supply chain operations. In addition, the Dataiku platform has the ability to share the dashboards with other stakeholders, such as procurement teams, logistics teams, or executives, to ensure everyone is aligned with the same goals and objectives.